using System;
using System.Collections.Generic;
using System.Xml;

namespace TopCoder.Uml.Xmi.MetaData
{
    /// <summary>
    /// Represnts an XMI element.  Other classes will extend this class to offer more specific details.  This class
    /// reads documentation and stereotypes.
    /// </summary>
    public class XmiElement
    {
        /// <summary>
        /// The id of the element.
        /// </summary>
        private string xmiId;

        /// <summary>
        /// List of UmlTaggedValue objects for documentation.
        /// </summary>
        private List<UmlTaggedValue> taggedValues = new List<UmlTaggedValue>();

        /// <summary>
        /// List of references to stereotypes.
        /// </summary>
        private List<string> stereotypeReferences = new List<string>();

        /// <summary>
        /// List of stereotypes.
        /// </summary>
        private List<string> stereotypes = new List<string>();

        /// <summary>
        /// Create a new XmiElement from an XmlNode.
        /// </summary>
        /// <param name="node">The node to read data from.</param>
        public XmiElement(XmlNode node)
        {
            this.xmiId = Helper.ReadAttribute(node, "xmi.id");
            this.LoadTaggedValues(node);
            this.ReadStereotypes(node);
        }

        /// <summary>
        /// Adds a stereotype to this XmiElement.
        /// </summary>
        /// <param name="stereotype">The stereotype to add.</param>
        public void AddStereotype(string stereotype)
        {
            this.stereotypes.Add(stereotype);
        }

        /// <summary>
        /// Get an array of stereotypes for this element.
        /// </summary>
        public string[] Stereotypes
        {
            get
            {
                return this.stereotypes.ToArray();
            }
        }

        /// <summary>
        /// Get the id for this element.
        /// </summary>
        public string XmiId
        {
            get
            {
                return this.xmiId;
            }
        }

        /// <summary>
        /// Get an array of id references to stereotypes.
        /// </summary>
        public string[] StereotypeReferences
        {
            get
            {
                return this.stereotypeReferences.ToArray();
            }
        }

        /// <summary>
        /// Get an array of UmlTaggedValue objects for documentation.
        /// </summary>
        public UmlTaggedValue[] UmlTaggedValues
        {
            get
            {
                return this.taggedValues.ToArray();
            }
        }

        /// <summary>
        /// Read stereotype references from an XmlNode.
        /// </summary>
        /// <param name="node">The node to read from.</param>
        private void ReadStereotypes(XmlNode node)
        {
            foreach (XmlNode childNode in node.ChildNodes)
            {
                foreach (XmlNode grandChildNode in childNode.ChildNodes)
                {
                    if (grandChildNode.Name == "UML:Stereotype")
                    {
                        string idRef = Helper.ReadAttribute(grandChildNode, "xmi.idref");
                        if (idRef != null)
                        {
                            this.stereotypeReferences.Add(idRef);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Read UML:TaggedValue (documentation) nodes.
        /// </summary>
        /// <param name="node">The XmlNode to read from.</param>
        private void LoadTaggedValues(XmlNode node)
        {
            foreach (XmlNode childNode in node.ChildNodes)
            {
                foreach (XmlNode subNode in childNode.ChildNodes)
                {
                    if (subNode.Name == "UML:TaggedValue")
                    {
                        UmlTaggedValue taggedValue = new UmlTaggedValue(subNode);
                        this.taggedValues.Add(taggedValue);
                    }
                }
            }
        }
    }
}