// Copyright (c) 2006, TopCoder, Inc. All rights reserved.
using System;
using System.Collections.Generic;
using NUnit.Framework;
using TopCoder.Uml.Xmi.MetaData;
using TopCoder.Uml.Xmi.Parser;

namespace TopCoder.Uml.Xmi
{
    /// <summary>
    /// Test class for XmiHandler
    /// </summary>
    /// <author>Dan Manges (Seizer)</author>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class XmiHandlerTest
    {
        /// <summary>
        /// XMI file to use for testing.
        /// </summary>
        private const string XMI_TEST_FILE = @"..\..\test_files\1.4.1\xsd2ddl9.xmi";

        /// <summary>
        /// Handler to use for testing.
        /// </summary>
        private XmiHandler xmiHandler;

        /// <summary>
        /// Setup testing for XmiHandler by parsing an XMI file.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            this.xmiHandler = new XmiHandler();

            XmiParser parser = new XmiParser();
            parser.AddHandler(XmiHandler.NAME, this.xmiHandler, XmiHandler.NODE_TYPES);
            parser.ParseFrom(XMI_TEST_FILE);
        }

        /// <summary>
        /// After testing, set variables to null.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            this.xmiHandler = null;
        }

        /// <summary>
        /// Test finding classes in the handler.
        /// </summary>
        [Test]
        public void TestClasses()
        {
            Assert.IsNotNull(this.FindClass("DdlGeneratorUtility"));
            Assert.IsNotNull(this.FindClass("IFileSource"));
            Assert.IsNotNull(this.FindClass("IDdlGenerator"));
            Assert.IsNotNull(this.FindClass("IStandard"));
            Assert.IsNotNull(this.FindClass("UnknownFileTypeException"));
            Assert.IsNotNull(this.FindClass("AbstractStandard"));
            Assert.IsNotNull(this.FindClass("AbstractDataValidationStandard"));
            Assert.IsNotNull(this.FindClass("ValidateTableName"));
            Assert.IsNotNull(this.FindClass("ValidateColumnName"));
            Assert.IsNotNull(this.FindClass("SimpleTypeMappingStandard"));
        }

        /// <summary>
        /// Test methods in classes.
        /// </summary>
        /// <remarks>
        /// This is being tested in the XmiHandler test because the handler does the method-to-class mapping.
        /// </remarks>
        [Test]
        public void TestMethods()
        {
            UmlClass umlClass = this.FindClass("DdlGeneratorUtility");
            Assert.AreEqual(1, this.ClassMethods(umlClass, "Main").Length);
            Assert.AreEqual(4, this.ClassMethods(umlClass, "DdlGeneratorUtility").Length);
            Assert.AreEqual(2, this.ClassMethods(umlClass, "GenerateDdl").Length);
            Assert.AreEqual(1, this.ClassMethods(umlClass, "ValidateConfigFilename").Length);
            Assert.AreEqual(1, this.ClassMethods(umlClass, "ValidateConfigNamespace").Length);
        }

        /// <summary>
        /// Test parameters in methods.
        /// </summary>
        [Test]
        public void TestParameters()
        {
            UmlClass umlClass = this.FindClass("IDdlGenerator");
            Method method = this.ClassMethods(umlClass, "GenerateDdl")[0];
            Assert.AreEqual(1, method.Parameters.Length);
            UmlParameter p = method.Parameters[0];
            Assert.AreEqual("dataSet", p.Name);
            Assert.AreEqual("DataSet", p.Type);

            Assert.IsNotNull(umlClass = this.FindClass("ExtendedXsdSource"));
            method = this.ClassMethods(umlClass, "DefineKeys")[0];
            Assert.AreEqual(2, method.Parameters.Length);
            Assert.AreEqual("element", method.Parameters[1].Name);
            Assert.AreEqual("XmlSchemaElement", method.Parameters[1].Type);
        }

        /// <summary>
        /// Test attributes in classes.
        /// </summary>
        [Test]
        public void TestAttributes()
        {
            UmlClass umlClass = this.FindClass("AndStandard");
            Assert.IsNotNull(umlClass);
            UmlAttribute umlAttribute = this.ClassAttribute(umlClass, "left");
            Assert.IsNotNull(umlAttribute);
            Assert.AreEqual("AbstractStandard", umlAttribute.Type);
            Assert.IsTrue(Array.IndexOf(umlAttribute.Stereotypes, "readonly") > -1);

            umlClass = this.FindClass("SimpleTypeMappingStandard");
            Assert.IsNotNull(umlClass);
            umlAttribute = this.ClassAttribute(umlClass, "Mapping");
            Assert.IsNotNull(umlAttribute);
            Assert.AreEqual("IDictionary", umlAttribute.Type);
            Assert.IsTrue(Array.IndexOf(umlAttribute.Stereotypes, "property-get") > -1);
        }

        /// <summary>
        /// Find a class in the handler.
        /// </summary>
        /// <param name="name">The class name.</param>
        /// <returns>The UmlClass instance or null if it cannot be found.</returns>
        private UmlClass FindClass(string name)
        {
            foreach (UmlClass umlClass in this.xmiHandler.Classes)
            {
                if (umlClass.Name == name)
                {
                    return umlClass;
                }
            }
            return null;
        }

        /// <summary>
        /// Find all methods with a given name in a given class.
        /// </summary>
        /// <param name="umlClass">The class containing the methods.</param>
        /// <param name="methodName">The method name to look for.</param>
        /// <returns>An array of matching methods.</returns>
        private Method[] ClassMethods(UmlClass umlClass, string methodName)
        {
            List<Method> methods = new List<Method>();
            foreach (Method method in umlClass.Methods)
            {
                if (method.Name == methodName)
                {
                    methods.Add(method);
                }
            }
            return methods.ToArray();
        }

        /// <summary>
        /// Find an attribute with the given name in a given class.
        /// </summary>
        /// <param name="umlClass">The class containing the attribute.</param>
        /// <param name="attributeName">The attribute name to look for.</param>
        /// <returns>The matching attribute.</returns>
        public UmlAttribute ClassAttribute(UmlClass umlClass, string attributeName)
        {
            UmlAttribute umlAttribute = null;
            foreach (UmlAttribute attribute in umlClass.UmlAttributes)
            {
                if (attribute.Name == attributeName)
                {
                    if (umlAttribute == null)
                    {
                        umlAttribute = attribute;
                    }
                    else
                    {
                        throw new ApplicationException("Double attribute name.");
                    }
                }
            }
            return umlAttribute;
        }
    }
}
